@@ -48,7 +48,7 @@ namespace Codeine
       m_thingy->setPaletteForegroundColor( paletteBackgroundColor().dark() );
 
       TQEvent e( TQEvent::Resize );
-      eventFilter( 0, &e );
+      eventFilter( nullptr, &e );
 
       adjustSize();
       show();
