@@ -18,11 +18,12 @@ class VolumeAction : public TDEToggleAction
    virtual int plug( TQWidget*, int );
 
 public slots:
-   void sliderMoved( int );
+   // Update Slider and Label to \a volume
+   void setVolume(int volume);
 
 private slots:
    void toggled( bool );
-   void sliderReleased() { setChecked( false ); toggled( false ); }
+   void sliderMoved(int);
 
 public:
    VolumeAction( TDEToolBar *anchor, TDEActionCollection *ac );
